// vincia25.cc is a part of the VINCIA plugin to PYTHIA 8
// This is a simple test program.
// It illustrates how Les Houches Event File input can be used in Pythia8.
// It uses the Wproduction.lhe input file, the latter with 5k events.
// Copyright (C) 2019 Peter Skands

// VINCIA include file
#include "Pythia8/Vincia.h"

// Use PYTHIA and VINCIA namespaces
using namespace Pythia8;

// Main Program.
int main() {

  // Generator.
  Pythia pythia;
  VinciaPlugin vincia(&pythia);

  // Read user settings from file.
  pythia.readFile("vincia25.cmnd");

  // Initialize Vincia & Pythia.
  if(!vincia.init()) { return EXIT_FAILURE; }

  // Allow abort of run if many errors.
  int  nAbort  = pythia.settings.mode("Main:timesAllowErrors");
  int  iAbort  = 0;
  bool doAbort = false;

  // Number of events.
  int nEvent = pythia.settings.mode("Main:numberOfEvents");

  // Begin event loop.
  for (int iEvent = 1; iEvent <= nEvent; ++iEvent) {

    // Generate next event.
    // Break out of event loop if at end of LHE file.
    if ( !pythia.next() ) {
      if ( pythia.info.atEndOfFile() ) break;
      else if (++iAbort > nAbort) { doAbort = true; break; }
      else continue;
    }
     
  } // End of event loop. 

  // Print infos.
  vincia.printInfo();
  pythia.stat();
  if (doAbort) 
    cout << endl << endl << "Aborting due to too many errors!" 
         << endl << endl;

  // Done.                           
  return 0;
}
